import es
import os
import time
import keyvalues
import gamethread
import langlib
import playerlib
import popuplib
import keymenulib

import psyco
psyco.full()

#plugin information
info = es.AddonInfo()
info.name = "Setting EventScripts python library"
info.version = "oy1b"
info.author = "Hunter"
info.url = "http://www.eventscripts.com/pages/Setting/"
info.description = "Provides user based setting handling for Source games"

#global variables:
## gSettings holds all the settings by their names for backwards compatibility
gSettings = {}

# Get this library handle for registering events dynamically:
selfmodule = __import__('settinglib')
# Get the data path for reading/writing KeyValues
selfdatapath = "%s/%s" % (es.server_var["eventscripts_addondir"], 'setting')
# Get the language strings
selflangpath = "%s/%s" % (es.server_var["eventscripts_addondir"], '_libs/python/settinglib')
selflangtext = langlib.Strings(selflangpath + "/strings.ini")



## Setting_setting class is the setting base class
class Setting_setting(object):
    def __init__(self, pSettingid, pDescription, pType="list", pFilename=None):
        #initialization of setting
        self.name = pSettingid          #setting name for backwards compatibility
        self.popup = {}                 #contains the popup object's per userid
        self.keyvalues = None           #contains the keyvalues object
        self.backmenuvar = 0            #backmenu that displays on 0. Cancel
        #init progress
        if pFilename:
            self.filename = pFilename
        else:
            self.filename = selfdatapath+'/'+self.name+'.txt'
        self.keyvalues = keyvalues.KeyValues(name=self.name)
        try:
            if os.path.exists(gSettings[pSettingid].filename):
                self.keyvalues.load(self.filename)
                if not self.keyvalues["config"]["name"] == self.name:
                    doinit = True
            else:
                doinit = True
        except KeyError:
            doinit = True
        if doinit:
            self.keyvalues["config"] = keyvalues.KeyValues(name="config")
            self.keyvalues["config"]["name"] = pSettingid
            self.keyvalues["config"]["descr"] = pDescription
            self.keyvalues["config"]["resend"] = True
            self.keyvalues["config"]["sound"] = None
            if pType == "list" or pType == "toggle":
                self.keyvalues["config"]["type"] = pType
            else:
                self.keyvalues["config"]["type"] = "list"
            self.keyvalues["options"] = keyvalues.KeyValues(name="options")
            self.keyvalues["users"] = keyvalues.KeyValues(name="users")
            self.keyvalues["variables"] = keyvalues.KeyValues(name="variables")
            self.keyvalues["variables"]["global"] = keyvalues.KeyValues(name="global")
            self.keyvalues.save(self.filename)
    def delete(self):
        #delete this setting, calls global delete function
        delete(self.name)
    def clear(self, timestamp=None):
        if timestamp:
            timediff = int(time.time()) - int(timestamp)
            for steamid in self.keyvalues["users"]:
                if int(str(self.keyvalues["users"][str(steamid)]["time"])) < int(timediff):
                    del self.keyvalues["users"][str(steamid)]
        else:
            for steamid in self.keyvalues["users"]:
                del self.keyvalues["users"][str(steamid)]
        for option in self.keyvalues["options"]:
            state = self.keyvalues["options"][str(option)]["state"]
            self.keyvalues["options"][str(option)]["globstate"] = bool(state)
        for steamid in self.keyvalues["variables"]:
            for variable in self.keyvalues["variables"][str(steamid)]:
                del self.keyvalues["variables"][str(steamid)][str(variable)]
    def save(self):
        self.keyvalues.save(self.filename)
    def saveToFile(self, filename, overwriteFilename=False):
        #this method is only for advanced users!
        self.keyvalues.save(filename)
        if overwriteFilename == True:
            self.filename = filename
    def loadFromFile(self, filename, overwriteFilename=False):
        #this method is only for advanced users!
        self.keyvalues.load(filename)
        if overwriteFilename == True:
            self.filename = filename
    def send(self, pUsers, prio=False, locked=False):
        try:
            # try if pUsers is just a single userid
            userid = int(pUsers)
            pUsers = [userid]
        except TypeError:
            pass
        if locked:
            optionstate = 0
        else:
            optionstate = 1
        for userid in pUsers:
            player = playerlib.getPlayer(userid)
            if player:
                steamid = playerlib.uniqueid(userid, True)
                helpline = selflangtext("setting "+self.keyvalues["config"]["type"]+" help", (), player.get("lang"))
                oldpage = popuplib.find("_setting_"+str(self.name)+"_user"+str(userid))
                if oldpage:
                    oldpage.delete()
                self.popup[userid] = popuplib.easymenu("_setting_"+str(self.name)+"_user"+str(userid), "_setting_choice", _submit)
                self.popup[userid].settingid = self.name
                self.popup[userid].c_titleformat = str(self.keyvalues["config"]["descr"]) + (" "*(30-len(str(self.keyvalues["config"]["descr"])))) + " (%p/%t)\n" + helpline
                if not steamid in self.keyvalues["users"]:
                    self.initUser(userid)
                if self.keyvalues["config"]["type"] == "list":
                    for option in self.keyvalues["options"]:
                        index = int(str(option))
                        display = self.keyvalues["options"][str(option)]["display"]
                        if self.keyvalues["users"][steamid]["data"][str(option)]["state"] == True:
                            active = selflangtext("setting list active", (), player.get("lang"))
                            self.popup[userid].addoption(index, display + " ("+active+")", optionstate)
                        else:
                            self.popup[userid].addoption(index, display, optionstate)
                elif self.keyvalues["config"]["type"] == "toggle":
                    for option in self.keyvalues["options"]:
                        index = int(str(option))
                        display = self.keyvalues["options"][str(option)]["display"]
                        if self.keyvalues["users"][steamid]["data"][str(option)]["state"] == True:
                            active = selflangtext("setting toggle on", (), player.get("lang"))
                            self.popup[userid].addoption(index, display + " ("+active+")", optionstate)
                        else:
                            active = selflangtext("setting toggle off", (), player.get("lang"))
                            self.popup[userid].addoption(index, display + " ("+active+")", optionstate)
                if self.backmenuvar:
                    self.popup[userid].submenu(10, self.backmenuvar)
                self.popup[userid].send(userid, prio)
    def sendglobal(self, pUsers, prio=False, locked=False):
        try:
            # try if pUsers is just a single userid
            userid = int(pUsers)
            pUsers = [userid]
        except TypeError:
            pass
        if locked:
            optionstate = 0
        else:
            optionstate = 1
        for userid in pUsers:
            player = playerlib.getPlayer(userid)
            if player:
                helpline = selflangtext("setting "+self.keyvalues["config"]["type"]+" help", (), player.get("lang"))
                oldpage = popuplib.find("_setting_"+str(self.name)+"_user"+str(userid)+"_global")
                if oldpage:
                    oldpage.delete()
                self.popup["global"] = popuplib.easymenu("_setting_"+str(self.name)+"_user"+str(userid)+"_global", "_setting_choice", _submitGlobal)
                self.popup["global"].settingid = self.name
                self.popup["global"].c_titleformat = str(self.keyvalues["config"]["descr"]) + (" "*(30-len(str(self.keyvalues["config"]["descr"])))) + " (%p/%t)\n" + helpline
                if self.keyvalues["config"]["type"] == "list":
                    for option in self.keyvalues["options"]:
                        index = int(str(option))
                        display = self.keyvalues["options"][str(option)]["display"]
                        if self.keyvalues["options"][str(option)]["globstate"] == True:
                            active = selflangtext("setting list active", (), player.get("lang"))
                            self.popup["global"].addoption(index, display + " ("+active+")", optionstate)
                        else:
                            self.popup["global"].addoption(index, display, optionstate)
                elif self.keyvalues["config"]["type"] == "toggle":
                    for option in self.keyvalues["options"]:
                        index = int(str(option))
                        display = self.keyvalues["options"][str(option)]["display"]
                        if self.keyvalues["options"][str(option)]["globstate"] == True:
                            active = selflangtext("setting toggle on", (), player.get("lang"))
                            self.popup["global"].addoption(index, display + " ("+active+")", optionstate)
                        else:
                            active = selflangtext("setting toggle off", (), player.get("lang"))
                            self.popup["global"].addoption(index, display + " ("+active+")", optionstate)
                if self.backmenuvar:
                    self.popup["global"].submenu(10, self.backmenuvar)
                self.popup["global"].send(userid, prio)
    def information(self, listlevel):
        if listlevel >= 1:
            es.dbgmsg(0, " ")
        es.dbgmsg(0, self.name)
        if listlevel >= 1:
            es.dbgmsg(0, "  Description:  "+str(self.keyvalues["config"]["descr"]))
            es.dbgmsg(0, "  Re-Send:      "+str(self.keyvalues["config"]["resend"]))
            es.dbgmsg(0, "  Soundfile:    "+str(self.keyvalues["config"]["sound"]))
        if listlevel >= 2:
            es.dbgmsg(0, "  Players:      "+str(len(self.keyvalues["users"].keys())))
            es.dbgmsg(0, "  Options:      "+str(len(self.keyvalues["options"].keys())))
            for option in self.keyvalues["options"]:
                es.dbgmsg(0,"    ["+str(option)+"]["+str(self.keyvalues["options"][str(option)]["keyname"])+"]:  "+self.keyvalues["options"][str(option)]["display"])
    def isValidOption(self, option):      #checks for valid options numbers
        if type(option) == int:
            if (option < 1): return False
            if (option > len(self.keyvalues["options"].keys())): return False
            return option
        else:
            for thisoption in self.keyvalues["options"]:
                if option == str(self.keyvalues["options"][str(thisoption)]["keyname"]):
                    return thisoption
            return False
    def initUser(self, userid):
        if es.exists('userid',userid):
            steamid = playerlib.uniqueid(userid, True)
            self.keyvalues["users"][steamid] = keyvalues.KeyValues(name=steamid)
            self.keyvalues["users"][steamid]["time"] = int(time.time())
            self.keyvalues["users"][steamid]["data"] = keyvalues.KeyValues(name="data")
            for option in self.keyvalues["options"]:
                self.keyvalues["users"][steamid]["data"][str(option)] = keyvalues.KeyValues(name=str(option))
                self.keyvalues["users"][steamid]["data"][str(option)]["state"] = self.keyvalues["options"][str(option)]["state"]
    def updateTime(self, userid):
        if es.exists('userid',userid):
            steamid = playerlib.uniqueid(userid, True)
            self.keyvalues["users"][steamid]["time"] = int(time.time())
    def menuUserSubmit(self, userid, value):
        if es.exists('userid',userid):
            if self.keyvalues["config"]["type"] == "list":
                self.set(int(value), userid)
            elif self.keyvalues["config"]["type"] == "toggle":
                self.toggle(int(value), userid)
            if self.keyvalues["config"]["sound"]:
                es.playsound(userid, self.keyvalues["config"]["sound"], 1.0)
            if self.keyvalues["config"]["resend"] == True:
                self.send(userid, True)
    def menuUserGlobalSubmit(self, userid, value):
        if es.exists('userid',userid):
            if self.keyvalues["config"]["type"] == "list":
                self.set(int(value))
            elif self.keyvalues["config"]["type"] == "toggle":
                self.toggle(int(value))
            if self.keyvalues["config"]["sound"]:
                es.playsound(userid, self.keyvalues["config"]["sound"], 1.0)
            if self.keyvalues["config"]["resend"] == True:
                self.sendglobal(userid, True)
    def settitle(self, text):             #set the description of the setting (old method name)
        self.setdescription(text)
    def setdescription(self, text):       #set the description of the setting
        self.keyvalues["config"]["descr"] = text
        self.keyvalues.save(self.filename)
    def resend(self, resend):
        if resend:
            self.keyvalues["config"]["resend"] = True
        else:
            self.keyvalues["config"]["resend"] = False
        self.keyvalues.save(self.filename)
    def setoption(self, option, text, state=None):    #edit/add an option
        option = str(self.isValidOption(option))
        if option:
            for thisoption in self.keyvalues["options"]:
                if int(str(thisoption)) == int(option):
                    self.keyvalues["options"][str(thisoption)]["display"] = text
                    if state != None and (state == True or state == False):
                        self.keyvalues["options"][str(thisoption)]["state"] = state
            self.keyvalues.save(self.filename)
        else:
            es.dbgmsg(0,"Settinglib: Cannot modify option #"+str(option)+"("+text+") in "+self.name)
    def addoption(self, key, text, state=None):            #add an option to the end of the list
        option = str(len(self.keyvalues["options"].keys())+1)
        self.keyvalues["options"][option] = keyvalues.KeyValues(name=option)
        self.keyvalues["options"][option]["keyname"] = key
        self.keyvalues["options"][option]["display"] = text
        if state != None and (state == True or state == False):
            self.keyvalues["options"][option]["state"] = state
            self.keyvalues["options"][option]["globstate"] = state
        else:
            self.keyvalues["options"][option]["state"] = False
            self.keyvalues["options"][option]["globstate"] = False
        self.keyvalues.save(self.filename)
    def deloption(self, option):          #delete an option
        arg = option
        option = str(self.isValidOption(option))
        if option:
            options = len(self.keyvalues["options"].keys())
            for thisoption in range(option,options):
                self.keyvalues["options"][str(thisoption)] = self.keyvalues["options"][thisoption+1]
            del self.keyvalues["options"][str(options)]
            for steamid in self.keyvalues["users"]:
                for thisoption in range(option,options):
                    self.keyvalues["users"][str(steamid)]["data"][str(thisoption)] = self.keyvalues["users"][str(steamid)]["data"][str(thisoption+1)]
                del self.keyvalues["users"][str(steamid)]["data"][str(options)]
            self.keyvalues.save(self.filename)
        else:
            raise IndexError("Settinglib: Cannot delete option %s, it does not exists"%arg)
    def clearoption(self):
        for option in self.keyvalues["options"]:
            self.deloption(option)
    def setdefault(self, option, arg1=None, arg2=None):
        arg = option
        option = str(self.isValidOption(option))
        if option:
            if self.keyvalues["config"]["type"] == "list":
                for thisoption in self.keyvalues["options"]:
                    if str(thisoption) == str(option):
                        self.keyvalues["options"][str(thisoption)]["state"] = True
                    else:
                        self.keyvalues["options"][str(thisoption)]["state"] = False
                if arg1:
                    for steamid in self.keyvalues["users"]:
                        for thisoption in self.keyvalues["users"][str(steamid)]["data"]:
                            if str(thisoption) == str(option):
                                self.keyvalues["users"][str(steamid)]["data"][str(thisoption)]["state"] = True
                            else:
                                self.keyvalues["users"][str(steamid)]["data"][str(thisoption)]["state"] = False
            elif self.keyvalues["config"]["type"] == "toggle":
                if arg1:
                    self.keyvalues["options"][str(option)]["state"] = True
                else:
                    self.keyvalues["options"][str(option)]["state"] = False
                if arg2:
                    for steamid in self.keyvalues["users"]:
                        if arg1:
                            self.keyvalues["users"][str(steamid)]["data"][option]["state"] = True
                        else:
                            self.keyvalues["users"][str(steamid)]["data"][option]["state"] = False
        else:
            raise IndexError("Settinglib: Cannot set option %s as default, it does not exists"%arg)
    def set(self, option, arg1=None, arg2=None):
        arg = option
        option = str(self.isValidOption(option))
        if option:
            if self.keyvalues["config"]["type"] == "list":
                if arg1:
                    steamid = playerlib.uniqueid(arg1, True)
                    if steamid:
                        for thisoption in self.keyvalues["users"][str(steamid)]["data"]:
                            if str(thisoption) == str(option):
                                self.keyvalues["users"][str(steamid)]["data"][str(thisoption)]["state"] = True
                            else:
                                self.keyvalues["users"][str(steamid)]["data"][str(thisoption)]["state"] = False
                    else:
                        raise IndexError("Settinglib: Cannot find userid %s"%arg1)
                else:
                    for thisoption in self.keyvalues["options"]:
                        if str(thisoption) == str(option):
                            self.keyvalues["options"][str(thisoption)]["globstate"] = True
                        else:
                            self.keyvalues["options"][str(thisoption)]["globstate"] = False
            elif self.keyvalues["config"]["type"] == "toggle":
                if arg2:
                    steamid = playerlib.uniqueid(arg2, True)
                    if steamid:
                        if arg1:
                            self.keyvalues["users"][str(steamid)]["data"][str(option)]["state"] = True
                        else:
                            self.keyvalues["users"][str(steamid)]["data"][str(option)]["state"] = False
                    else:
                        raise IndexError("Settinglib: Cannot find userid %s"%arg2)
                else:
                    if arg1:
                        self.keyvalues["options"][str(option)]["globstate"] = True
                    else:
                        self.keyvalues["options"][str(option)]["globstate"] = False
        else:
            raise IndexError("Settinglib: Cannot set option #%s, it does not exists"%arg)
    def get(self, option=None, arg1=None):
        if self.keyvalues["config"]["type"] == "list":
            if option:
                steamid = playerlib.uniqueid(option, True)
                if steamid:
                    for thisoption in self.keyvalues["users"][str(steamid)]["data"]:
                        if str(self.keyvalues["users"][str(steamid)]["data"][str(thisoption)]["state"]) == "1":
                            return str(self.keyvalues["options"][str(thisoption)]["keyname"])
                else:
                    raise IndexError("Settinglib: Cannot find userid %s"%option)
            else:
                for thisoption in self.keyvalues["options"]:
                    if str(self.keyvalues["options"][str(thisoption)]["globstate"]) == "1":
                        return str(self.keyvalues["options"][str(thisoption)]["keyname"])
        elif self.keyvalues["config"]["type"] == "toggle":
            arg = option
            option = str(self.isValidOption(option))
            if option:
                if arg1:
                    steamid = playerlib.uniqueid(arg1, True)
                    if steamid:
                        try:
                            result = bool(int(str(self.keyvalues["users"][str(steamid)]["data"][str(option)]["state"])))
                        except KeyError:
                            result = bool(int(str(self.keyvalues["options"][str(option)]["state"])))
                        return result
                    else:
                        raise IndexError("Settinglib: Cannot find userid %s"%arg2)
                else:
                    return bool(int(str(self.keyvalues["options"][str(option)]["globstate"])))
            else:
                raise IndexError("Settinglib: Cannot get option #%s, it does not exists"%arg)
    def toggle(self, option, userid=None):
        arg = option
        option = self.isValidOption(option)
        if option:
            if self.keyvalues["config"]["type"] == "toggle":
                state = self.get(option, userid)
                if state:
                    self.set(option, False, userid)
                else:
                    self.set(option, True, userid)
            else:
                raise TypeError("Settinglib: Can't toggle a setting with type 'list'")
        else:
            raise IndexError("Settinglib: Cannot toggle option #%s, it does not exists"%arg)
    def addsound(self, sound):            #add a sound file
        self.keyvalues["config"]["sound"] = sound
        self.keyvalues.save(self.filename)
    def delsound(self):                   #delete a sound file
        self.keyvalues["config"]["sound"] = None
        self.keyvalues.save(self.filename)
    def setvar(self, variable, value, userid=None):
        if userid:
            if es.exists('userid',userid):
                steamid = playerlib.uniqueid(userid, True)
            else:
                raise IndexError("Settinglib: Cannot find userid '%s', it does not exists"%userid)
        else:
            steamid = "global"
        self.keyvalues["variables"][str(steamid)][str(variable)] = value
        self.keyvalues.save(self.filename)
    def getvar(self, variable, userid=None):
        if userid:
            if es.exists('userid',userid):
                steamid = playerlib.uniqueid(userid, True)
            else:
                raise IndexError("Settinglib: Cannot find userid '%s', it does not exists"%userid)
        else:
            steamid = "global"
        return self.keyvalues["variables"][str(steamid)][str(variable)]
    def backmenu(self, backmenu):
        if popuplib.exists(backmenu):
            for page in self.popup:
                self.popup[page].submenu(10, backmenu)
            self.backmenuvar = backmenu
            es.dbgmsg(1, "Settinglib: Set backmenu of '"+str(self.name)+"' to popup '"+str(self.backmenuvar)+"'")
            return True
        elif keymenulib.exists(backmenu):
            keymenu = keymenulib.find(backmenu)
            for page in self.popup:
                self.popup[page].submenu(10, keymenu.popup.name)
            self.backmenuvar = keymenu.popup.name
            es.dbgmsg(1, "Settinglib: Set backmenu of '"+str(self.name)+"' to popup '"+str(self.backmenuvar)+"'")
            return True
        else:
            es.dbgmsg(0, "setting: Could not set backmenu of '"+str(self.name)+"' to '"+str(backmenu)+"'")
            return False

#setting commands begin here
#usage from other Python scripts for example:
#  import es
#  import setting
#  a = setting.create("insertnamehere", "My Setting", "list")
def create(pSettingid, pDescription, pType="list", pFilename=None):
    #create new setting
    gSettings[pSettingid] = Setting_setting(pSettingid, pDescription, pType, pFilename)
    return gSettings[pSettingid]
    
def free(pSettingid):
    #free a setting
    if (pSettingid in gSettings):
        if os.path.exists(gSettings[pSettingid].filename):
            os.remove(gSettings[pSettingid].filename)
        del gSettings[pSettingid]
    else:
        raise ValueError("Settinglib: Cannot free setting %s, it does not exists"%pSettingid)

def delete(pSettingid):
    #delete a setting
    if (pSettingid in gSettings):
        del gSettings[pSettingid]
    else:
        raise ValueError("Settinglib: Cannot delete setting %s, it does not exists"%pSettingid)

def exists(pSettingid):
    #does named setting exist
    return (pSettingid in gSettings)

def find(pSettingid):
    #return class instance of named popup
    if (pSettingid in gSettings):
        return gSettings[pSettingid]
    return None

def send(pSettingid, pUserid, prio=False):
    #send a named setting to user/users
    if pSettingid in gSettings:
        gSettings[pSettingid].send(pUserid,prio)
    else:
        raise ValueError("Settinglib: Cannot send setting %s, it does not exists"%pSettingid)

def sendGlobal(pSettingid, pUserid, prio=False):
    #send a named setting to user/users
    if pSettingid in gSettings:
        gSettings[pSettingid].sendGlobal(pUserid,prio)
    else:
        raise ValueError("Settinglib: Cannot send setting %s, it does not exists"%pSettingid)

###################
#Helper functions #
###################

def _submit(userid, value, popupid):
    es.dbgmsg(1, "Settinglib: _submit("+str(userid)+", "+str(value)+", "+str(popupid)+")")
    page = popuplib.find(popupid)
    if page.settingid in gSettings:
        gSettings[page.settingid].menuUserSubmit(userid, value)
    else:
        raise ValueError("Settinglib: Invalid setting %s submitted from popup"%page.settingid)

def _submitGlobal(userid, value, popupid):
    es.dbgmsg(1, "Settinglib: _submitGlobal("+str(userid)+", "+str(value)+", "+str(popupid)+")")
    page = popuplib.find(popupid)
    if page.settingid in gSettings:
        gSettings[page.settingid].menuUserGlobalSubmit(userid, value)
    else:
        raise ValueError("Settinglib: Invalid setting %s submitted from popup"%page.settingid)


def _saveTicker():
    gamethread.delayed(300, _saveTicker, ())
    for setting in gSettings:
        gSettings[setting].save()
        
def _saveAll():
    for setting in gSettings:
        gSettings[setting].save()
                    
######################
#EventScripts events #
######################

def es_map_start(event_var):
    gamethread.delayed(300, _saveTicker, ())    

def player_activate(event_var):
    userid = int(event_var['userid'])
    steamid = playerlib.uniqueid(userid, True)
    for setting in gSettings:
        if not steamid in gSettings[setting].keyvalues["users"]:
            gSettings[setting].initUser(userid)
        gSettings[setting].updateTime(userid)

# Register for custom events
es.addons.registerForEvent(selfmodule, 'es_map_start', es_map_start)
es.addons.registerForEvent(selfmodule, 'player_activate', player_activate)
